/* ==============================================================[ ap-lirc.c ]=
 * ap-lirc.cc - LIRC interface plugin main file
 * Control AlsaPlayer through LIRC.
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This program allows one to control AlsaPlayer using a remote control through
 * LIRC (Linux Infrared Remote Control).
 *
 * Lots of code ripped from irexec.c and the AlsaPlayer text and gtk interface
 * plugins.  Commands based on lirc-xmms-plugin.
 *
 * ------------------------------------------------------------------------- */

#include <iostream>

#include <alsaplayer/AlsaPlayer.h>
#include <alsaplayer/SampleBuffer.h>
#include <alsaplayer/CorePlayer.h>
#include <alsaplayer/Playlist.h>
#include <alsaplayer/alsaplayer_error.h>
#include <alsaplayer/interface_plugin.h>

#include <lirc/lirc_client.h>

//#define DEBUG

int interface_lirc_init()
{
  return 1;
}

int interface_lirc_running()
{
  return 1;
}

int interface_lirc_stop()
{
  return 1;
}

void interface_lirc_close()
{
  return;
}

int interface_lirc_start(Playlist *playlist, int argc, char **argv)
{
#ifdef DEBUG
  std::cout << "In lirc_start\n";
#endif DEBUG

  if(lirc_init("alsaplayer",1)==-1)
  {
    std::cerr << "lirc interface error: could not initialize lirc\n";
    return 1;
  }

  struct lirc_config *config;

  if(lirc_readconfig(NULL,&config,NULL)!=0)
  {
    std::cerr << "lirc interface error: could not read configuration\n";
    lirc_deinit();
    return 1;
  }

  CorePlayer *coreplayer = playlist->GetCorePlayer();

  char *code;
  char *c;
  char *args;
  int arg_int;
  int ret;
  float oldspeed;

  while(lirc_nextcode(&code)==0)
  {
    if(code==NULL) continue;
    while((ret=lirc_code2char(config,code,&c))==0 &&
	  c!=NULL)
    {
      int i;
      for (i = 0; c[i] != '\0'; i++)
	c[i] = tolower(c[i]);
#ifdef DEBUG
      std::cout << "command:" << c << endl;
#endif

      c = strdup(c);
      /* separate command from arguments -- split at the first space */
      if ((args = strchr(c, ' ')) != NULL)
      {
	*args = '\0';
	args++;
	while (*args == ' ')
	  args++;
	arg_int = atoi(args);
      }

      /* big, huge, if-else !!! */
      if (strcmp(c, "play") == 0)
	playlist->UnPause();
      else if (strcmp(c, "stop") == 0)
	playlist->Stop();
      else if (strcmp(c, "pause") == 0)
      {
	oldspeed = coreplayer->GetSpeed();
	coreplayer->SetSpeed(0.0);
      }
      else if (strcmp(c, "playpause") == 0)
      {
	if (coreplayer->GetSpeed() == 0.0)
	{
	  coreplayer->SetSpeed(oldspeed);
	}
	else
	{
	  oldspeed = coreplayer->GetSpeed();
	  coreplayer->SetSpeed(0.0);
	}
      }
      else if (strcmp(c, "next") == 0)
      {
	if (args)
	  for (i = 0; i < arg_int; i++)
	    playlist->Next();
	else
	  playlist->Next();
      }
      else if (strcmp(c, "prev") == 0)
      {
	if (args)
	  for (i = 0; i < arg_int; i++)
	    playlist->Prev();
	else
	  playlist->Prev();
      }
      else if (strcmp(c, "quit") == 0)
      {
	free(code);
	lirc_freeconfig(config);
	lirc_deinit();
	return 0;
      }
      else if (strcmp(c, "playlist_clear") == 0)
	playlist->Clear();
      else if (strcmp(c, "fwd") == 0)
      {
	if (!args)
	  arg_int = 1;
	int cur_pos = (coreplayer->GetCurrentTime() / 100) + arg_int;
	if (cur_pos < 0)
	  cur_pos = 0;
	cur_pos *= coreplayer->GetSampleRate();
	cur_pos /= coreplayer->GetFrameSize();
	cur_pos *=  coreplayer->GetChannels();
	cur_pos *= 2; // 16-bit = 2 * 1 byte
	coreplayer->Seek(cur_pos);
      }
      else if (strcmp(c, "bwd") == 0)
      {
	if (!args)
	  arg_int = 1;
	int cur_pos = (coreplayer->GetCurrentTime() / 100) - arg_int;
	if (cur_pos < 0)
	  cur_pos = 0;
	cur_pos *= coreplayer->GetSampleRate();
	cur_pos /= coreplayer->GetFrameSize();
	cur_pos *=  coreplayer->GetChannels();
	cur_pos *= 2; // 16-bit = 2 * 1 byte
	coreplayer->Seek(cur_pos);
      }
      else if (strcmp(c, "vol_up") == 0)
      {
	int volume;
	if (!args)
	  arg_int = 5;
	volume = coreplayer->GetVolume();
	volume += arg_int;
	if (volume > 100)
	  volume = 100;
	else if (volume < 0)
	  volume = 0;
	coreplayer->SetVolume(volume);
      }
      else if (strcmp(c, "vol_down") == 0)
      {
	int volume;
	if (!args)
	  arg_int = -5;
	volume = coreplayer->GetVolume();
	volume += arg_int;
	if (volume > 100)
	  volume = 100;
	else if (volume < 0)
	  volume = 0;
	coreplayer->SetVolume(volume);
      }
      else if (strcmp(c, "pan_left") == 0)
      {
	int pan;
	if (!args)
	  arg_int = 5;
	pan = coreplayer->GetPan();
	pan -= arg_int;
	if (pan > 100)
	  pan = 100;
	else if (pan < -100)
	  pan = 0;
	coreplayer->SetPan(pan);
      }
      else if (strcmp(c, "pan_right") == 0)
      {
	int pan;
	if (!args)
	  arg_int = 5;
	pan = coreplayer->GetPan();
	pan += arg_int;
	if (pan > 100)
	  pan = 100;
	else if (pan < -100)
	  pan = 0;
	coreplayer->SetPan(pan);
      }
      else if (strcmp(c, "pan_center") == 0)
	coreplayer->SetPan(0);
      else if (strcmp(c, "faster") == 0)
      {
	float speed;
	if (!args)
	  arg_int = 20;
	speed = coreplayer->GetSpeed();
	speed += (double) arg_int / 100.0;
	coreplayer->SetSpeed(speed);
      }
      else if (strcmp(c, "slower") == 0)
      {
	float speed;
	if (!args)
	  arg_int = 20;
	speed = coreplayer->GetSpeed();
	speed -= (double) arg_int / 100.0;
	coreplayer->SetSpeed(speed);
      }
      else if (strcmp(c, "speed") == 0)
      {
	if (!args)
	  arg_int = 100;
	coreplayer->SetSpeed((double) arg_int / 100.0);
      }

      free(c);
    }

    free(code);
    if(ret==-1) break;
  }

#ifdef DEBUG
  std::cout << "Exiting lirc_start\n";
#endif DEBUG

  lirc_freeconfig(config);
  lirc_deinit();
  return 0;
}

interface_plugin default_plugin =
{
  INTERFACE_PLUGIN_VERSION,
  "LIRC interface v0.4",
  "Hubert Chan",
  NULL,
  interface_lirc_init,
  interface_lirc_start,
  interface_lirc_running,
  interface_lirc_stop,
  interface_lirc_close
};

extern "C" {

interface_plugin *interface_plugin_info()
{
  return &default_plugin;
}

}

// end of ap-lirc.cc
